from django.contrib.auth import get_user_model


class MobileAuthBackend(object):
    """Allows user to sign-in using email, username or phone_number."""

    def authenticate(self, username=None, password=None, **kwargs):
        UserModel = get_user_model()

        try:
            if '@' in username:
                user = UserModel._default_manager.get(email=username)
            elif '+' in username[0]:
                user = UserModel._default_manager.get(phone_number=username)
            else:
                user = UserModel._default_manager.get(username=username)
        except UserModel.DoesNotExist:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a non-existing user (#20760).
            UserModel().set_password(password)
        else:
            if user.check_password(password) and self.user_can_authenticate(user):
                return user

    def user_can_authenticate(self, user):
        """Reject users with is_active=False. Custom user models that don't have that attribute are allowed."""
        is_active = getattr(user, 'is_active', None)
        return is_active or is_active is None

    def get_user(self, user_id):
        UserModel = get_user_model()
        try:
            user = UserModel._default_manager.get(pk=user_id)
        except UserModel.DoesNotExist:
            return None
        return user if self.user_can_authenticate(user) else None



from __future__ import unicode_literals

from django.contrib.auth.models import (AbstractBaseUser, BaseUserManager,
                                        PermissionsMixin)
from django.core.mail import send_mail
from django.db import models
from django.utils import six, timezone
from django.utils.translation import ugettext_lazy as _

from mobile_auth.fields import EmailNullField, PhoneNumberNullField
from mobile_auth.validators import (ASCIIUsernameValidator,
                                    UnicodeUsernameValidator)


class MobileUserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, username, email, password, phone_number, **extra_fields):
        """Creates and saves a User with the given username, email and password."""
        if not username:
            raise ValueError('The given username must be set')
        email = self.normalize_email(email)
        phone_number = phone_number
        username = self.model.normalize_username(username)
        user = self.model(username=username, email=email, phone_number=phone_number, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, username, email=None, phone_number=None, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        return self._create_user(username, email, password, phone_number=phone_number, **extra_fields)

    def create_superuser(self, username, password, email=None, phone_number=None, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self._create_user(username, email, password, phone_number=phone_number, **extra_fields)


class AbstractMobileUser(AbstractBaseUser, PermissionsMixin):
    """
    An abstract base class implementing a fully featured User model with admin-compliant permissions.

    Username and password are required. Other fields are optional.
    """

    username_validator = UnicodeUsernameValidator() if six.PY3 else ASCIIUsernameValidator()
    username = models.CharField(
        _('username'),
        max_length=50,
        unique=True,
        help_text=_('Required. 50 characters or fewer. Letters, digits and ./-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)
    email = EmailNullField(_('email address'), max_length=255, blank=True, null=True, unique=True)
    phone_number = PhoneNumberNullField(_('phone number'), blank=True, null=True, unique=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = MobileUserManager()

    USERNAME_FIELD = 'username'  # It should be ['username', 'email', 'phone_number']
    # REQUIRED_FIELDS = []

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def get_full_name(self):
        """Return the first_name plus the last_name, with a space in between."""
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        """Return the short name for the user."""
        return self.first_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Sends an email to this User."""
        send_mail(subject, message, from_email, [self.email], **kwargs)


class MobileUser(AbstractMobileUser):

    class Meta(AbstractMobileUser.Meta):
        swappable = 'AUTH_USER_MODEL'



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import ugettext_lazy as _

from mobile_auth.models import MobileUser


@admin.register(MobileUser)
class MobileUserAdmin(UserAdmin):

    fieldsets = (
        (None, {'fields': ('username', 'email', 'phone_number', 'password')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )

    list_display = ('username', 'email', 'phone_number', 'is_staff')
    search_fields = ('username', 'email', 'phone_number')



from django.db import models

from phonenumber_field.modelfields import PhoneNumberField


class EmailNullField(models.EmailField):
    description = "EmailField that stores NULL"

    def get_db_prep_value(self, value, connection=None, prepared=False):
        value = super(EmailNullField, self).get_db_prep_value(value, connection, prepared)
        if value == "":
            return None
        else:
            return value


class PhoneNumberNullField(PhoneNumberField):
    description = "PhoneNumberField that stores NULL when empty"

    def get_db_prep_value(self, value, connection=None, prepared=False):
        value = super(PhoneNumberNullField, self).get_db_prep_value(value, connection, prepared)
        if value == "":
            return None
        else:
            return value



#!/usr/bin/env python

from io import open
from setuptools import setup, find_packages


setup(
    name='django_mobile_auth',
    version='0.2.6',
    packages=find_packages(),
    license='MIT',
    author='Ricardo S. A. Silva',
    description='A Django app that allows users to authenticate using email, phone number or username.',
    keywords='django auth email phone',
    author_email='ricardo@salamandra.cc',
    long_description=open('README.md').read(),
    install_requires=['django', 'django-phonenumber-field==1.3.0', 'phonenumberslite==8.3.2'],
    include_package_data=True,
    url='https://github.com/ricardosasilva/django_mobile_auth',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ]
)
